package com.csc.library.components;

import org.apache.commons.lang.StringUtils;

import com.csc.library.database.FieldManager;
import com.csc.library.download.FileDownloadUtils2;
import com.csc.library.utilities.MyLog;

/**
 * 
 * Fileupload Ѻ upload
 * <br />
 * <h5>How ho use</h5>
 * <pre>
 * &lt;PICTURE lang=&quot;A&quot; encrypt=&quot;false&quot; log=&quot;false&quot; type=&quot;String&quot;&gt;
 *	  &lt;WIDTH&gt;30&lt;/WIDTH&gt;
 *	  &lt;DESC_ENG&gt;Picture&lt;/DESC_ENG&gt;
 *	  &lt;DESC_THAI&gt;ٻҾ&lt;/DESC_THAI&gt;
 *	  &lt;COMPONENT type=&quot;fileupload&quot;&gt;
 *	    &lt;SIZE&gt;20&lt;/SIZE&gt;
 *	    &lt;ALT_ENG&gt;Picture&lt;/ALT_ENG&gt;
 *	    &lt;ALT_THAI&gt;ٻҾ&lt;/ALT_THAI&gt;
 *	    &lt;READONLY&gt;true&lt;/READONLY&gt;
 *	    &lt;DISPLAY_LINK&gt;0&lt;/DISPLAY_LINK&gt;
 *	  &lt;/COMPONENT&gt;
 * &lt;/PICTURE&gt;
 *</pre>
 *
 * @version 2
 * 
 * @author James
 * 
 * @require <ul>
 *                  <li><a href="http://commons.apache.org/lang/">Apache Commons Lang</a></li>
 *                  <li>CscFileUtils</li>
 *                </ul>
 *
 */
public class Fileupload extends MyTag {

	private static final long serialVersionUID = -6417325453936606241L;
	private String tagInput = "";
	private String hspace = "";
	
	protected String uploadButton = "";
	protected String deleteButton = "";
	protected String downloadLink = "";
	
	protected String uploadField;
	protected String displayLink = "1";

	public Fileupload() {
		this.type = "fileupload";
	}

	public Fileupload(ComponentManager cm) {
		super(cm);
		this.ini();
	}

	public void ini() {
		
		setProperties();
		genUploadButton();
		genDeleteButton();
		
		if( !"0".equalsIgnoreCase(displayLink) ) {
			genDownloadLink();
		}
				
		// generate inputbox
		StringBuilder html = new StringBuilder();
		html.append("<input type=\"hidden\"  ");
		html.append("name=\"").append( name ).append("\" ");
		html.append("componenttype=\"").append( "fileupload" ).append("\" ");
		html.append("value=\"").append( value ).append("\" ");
		html.append(" />");
		
		html.append( this.uploadButton );
		html.append( this.hspace );
		html.append( deleteButton );
		html.append( this.hspace ).append("&nbsp;&nbsp;");
		html.append(downloadLink);
		
		tagInput = html.toString();
	}

	public String process() {
		ini();
		return tagInput;
	}

	public String getInput() {
		return tagInput;
	}

	public void setSpace(int space) {
		for (int i = 0; i < space; i++) {
			this.hspace = this.hspace + "&nbsp;";
		}
	}
		
	private void genUploadButton() {
		
		StringBuilder html = new StringBuilder();
		html.append("<a href=\"javascript:;\" onClick=\"uploadFile('").append( uploadField ).append("'");
		html.append(", '").append( name ).append("'");
		html.append(")\" >");
		html.append("<img id=\"").append(name).append("_upload\" src=\"../IMAGES/BUTTON/MAIN/UPOAD20.gif\" ");
		html.append("width=\"20\" height=\"20\" border=\"0\" alt=\"Insert Picture\"></a>");
		
		uploadButton= html.toString();
	}
	
	private void genDeleteButton() {
		
		StringBuilder html = new StringBuilder();
		html.append("<a href=\"javascript:;\" onClick=\"deleteFile('").append( uploadField ).append("'");
		html.append(", '").append( name ).append("'");
		html.append(", '").append( value ).append("'");
		html.append(")\" >");
		html.append("<img id=\"").append(name).append("_delete\" src=\"../IMAGES/BUTTON/MAIN/DEL20.gif\" ");
		html.append("width=\"20\" height=\"20\" border=\"0\" alt=\"Delete Picture\"></a>");
		
		deleteButton = html.toString();
	}
	
	private void genDownloadLink() {
		if(StringUtils.isNotBlank(value)) {
			StringBuilder buffer = new StringBuilder();
			buffer.append("<a id=\"").append(name).append("_link\" class=\"downloadLink\" href=\"");
			
			FileDownloadUtils2 utils = new FileDownloadUtils2();
			utils.setUploadField(uploadField);
			utils.setFilename(value);
			
			buffer.append(utils.getDownloadUrl());
			buffer.append("\" >").append(value).append("</a>");
			
			downloadLink = buffer.toString();
		}
	}
	
	private void setProperties() {
		
		try {
			
			FieldManager fm = (FieldManager)com;
			uploadField = fm.getParent().getTableName() +"." +fm.getName();
			
			String _displayLink = com.getProperty("displaylink");	
			this.displayLink = StringUtils.trimToEmpty(_displayLink);

		} catch (Exception e) {
			MyLog.error(this, e);
		}
			
	}

}
