package com.csc.library.database;

import java.math.BigDecimal;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Hashtable;
import java.util.Vector;

import com.csc.library.session.DbRecord;
import com.csc.library.session.DbTable;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscException;

public interface TableManager {
	public void refresh();
	public void next();
	public void previous();
	public void first();
	public void last();
	public int recCount();
	public int recNo();
	public boolean isFirst();
	public boolean isLast();
	public boolean eof();
	public void clear();
	public int save();
	public int delete();
	//public int save(Connection myCon);
	//public int delete(Connection myCon);
	public void restore();
	public void print();
	public void printAll();
	public String getString(String fieldName);
	public boolean getBoolean(String fieldName);
	public byte getByte(String fieldName);
	public char[] getChars(String fieldName);
	public char getChar(String fieldName);
	public short getShort(String fieldName);
	public int getInt(String fieldName);
	public long getLong(String fieldName);
	public float getFloat(String fieldName);
	public double getDouble(String fieldName);
	public Timestamp getTimestamp(String fieldName);
	public BigDecimal getBigcimal(String fieldName);
	public Time getTime(String fieldName);
	public java.sql.Date getDate(String fieldName);

	public boolean set(String fieldName, String data);
	public boolean set(String fieldName, boolean data);
	public boolean set(String fieldName, byte data);
	public boolean set(String fieldName, char data[]);
	public boolean set(String fieldName, char data);
	public boolean set(String fieldName, short data);
	public boolean set(String fieldName, int data);
	public boolean set(String fieldName, long data);
	public boolean set(String fieldName, float data);
	public boolean set(String fieldName, double data);
	public boolean set(String fieldName, Timestamp data);
	public boolean set(String fieldName, java.sql.Date data);
	public boolean set(String fieldName, BigDecimal data);
	public boolean set(String fieldName, Time data);

	public String getError();
	public void setCondition();

	public void setKey(String data);
	public void setKey(String data[]);
	public boolean setColumn(String columnList);
	public String getListKey();
	public void setUsedFor(char usedFor);
	public char getUsedFor();

	public String getTableName();
	public void setTableName(String name);
	public void setActualName(String name);
	public int setCondition(String condition[]);
	public void search();

	public void addRecord();
	public DbRecord addRecord(String key) throws CscException;
	public void deleteRecord();
	public void deleteRecord(int Recno);

	public Hashtable getChildTable();
	public void setChildTable(Hashtable hashChild);
	public Hashtable getRelationInfo();
	public void setRelationInfo(Hashtable hashRelation);
	public void putRelation(Relation oRela);
	public void removeRelation(String tableName);
	public Relation getRelation(String tableName);
	public int putChild(ChildManager oChild);
	public int putChild(String childName, char type, char usedFor);
	public void removeChild(String childName);
	public void refreshChild();
	public void refreshChild(String childName);
	public ChildManager getChild(String childName);
	public DbRecord getRecordManager(String childName);
	public DbRecord getCurrentRecordManager();
	public DbRecord getCurrentRecordManager(int recNo);
	public DbTable getTableManager(String childName);
	public String listChild();
	public void setAutoRefresh(byte status);
	public byte getAutoRefresh();
	public Hashtable getEffectTable();
	public void setEffectTable(Hashtable hashEffectTable);
	public int setMinorKey(String keyName);
	public String getMinorKey();
	public void setKeyOff();
	public void setKeyOn();
	public boolean getKeyStatus();

	public void setPropDB(String comp, String user, String pwd);
	public String getFieldList();

	public CscCalendar getCscCalendar(String fieldName);
	public String getLastUpdate();

	public void setOrderBy(String o);
	//public void setFilter(String s);
	public DbRecord getNewRecord();
	public void putNewRecord(DbRecord rec);
	public void setLastUpdate();

	public int fieldGroupCount();
	public Vector getAllFieldGroupName();
	public void putFieldGroup(FieldGroup gf);
	public FieldGroup getFieldGroup(String groupCode);
	public String getLinkFieldChild(String childName);
	public String getLinkFieldParent(String childName);
	public boolean setColumn(FieldGroup fg);
	public boolean addChildColumn(String childName);
	public void setFieldGroupColumn(String groupCode);
	public boolean addColumn(String columnList);

	public void setRefreshModel(byte model);

	public void setJSPString(String fieldName, String data);
	public String getJSPString(String fieldName);
	public void setConnectControl(ConnectManager con);
	public ConnectManager getConnectControl();
	public void setAllRequiredFound(boolean require);
	public void setRequiredFound(boolean require);
	public boolean getRequiredFound();
}
