package com.csc.library.mail;

import java.io.InputStream;
import java.util.Date;
import java.util.StringTokenizer;

import javax.mail.Address;
import javax.mail.FetchProfile;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Store;

import com.csc.library.utilities.MyLog;

/*
 * Package main MailBox
 * Handle 'read' and 'delete' messages
 */

public class MyMail {
	private Store store;
	private Folder folder;
	public MyMail(String hostname, String username, String password) {
		MailConnect con = new MailConnect(hostname, username, password);
		store = con.getStore();
	}

	public int read(int msgNo) {
		try {
			if (this.openMailBox("inbox") == 1) {
				int totalMessages = folder.getMessageCount();
				int newMessages = folder.getNewMessageCount();
				MyLog.info(this, "Total message: " + totalMessages);
				MyLog.info(this, "Unread message: " + newMessages);
				if (msgNo == -1) {
					// Attributes & Flags for all messages ..
					Message[] msgs = folder.getMessages();
					// Use a suitable FetchProfile
					FetchProfile fp = new FetchProfile();
					fp.add(FetchProfile.Item.ENVELOPE);
					fp.add(FetchProfile.Item.FLAGS);
					fp.add("X-Mailer");
					folder.fetch(msgs, fp);
					for (int i = 0; i < msgs.length; i++) {
						MyLog.info(this, "--------------------------");
						MyLog.info(this, "MESSAGE #" + (i + 1) + ":");
						dumpEnvelope(msgs[i]);
						// dumpPart(msgs[i]);
					}
				} else {
					MyLog.info(this, "Getting message number: " + msgNo);
					Message m = folder.getMessage(msgNo);
					dumpPart(m);
				}
				this.closeMailBox();
				return 1;
			} else
				return 0;
		} catch (Exception ex) {
			MyLog.error(this, ex);
			return 0;
		}
	}

	public void delete(String msgNo) {
		try {
			if (this.openMailBox("inbox") == 1) {
				for (StringTokenizer token = new StringTokenizer(msgNo, ","); token.hasMoreTokens();) {
					Message msg = folder.getMessage(Integer.parseInt(token.nextToken()));
					msg.setFlag(Flags.Flag.DELETED, true); //set delete flag
				}
				this.closeMailBox();
			}
		} catch (Exception ex) {
			MyLog.error(this, ex);
		}
	}

	public int delete(int msgNo) {
		try {
			if (this.openMailBox("inbox") == 1) {
				Message msg = folder.getMessage(msgNo);
				msg.setFlag(Flags.Flag.DELETED, true); //set delete flag
				this.closeMailBox();
				return 1;
			} else
				return 0;
		} catch (Exception ex) {
			MyLog.error(this, ex);
			return 0;
		}
	}

	public int messageList() {
		return read(-1);
	}

	//show message content
	private void dumpPart(Part p) throws Exception {
		if (p instanceof Message)
			dumpEnvelope((Message) p);
		MyLog.info(this, "CONTENT-TYPE: " + p.getContentType());
		Object o = p.getContent();
		if (o instanceof String) {
			MyLog.info(this, "This is a String");
			MyLog.info(this, (String) o);
		} else if (o instanceof Multipart) {
			MyLog.info(this, "This is a Multipart");
			Multipart mp = (Multipart) o;
			int count = mp.getCount();
			for (int i = 0; i < count; i++)
				dumpPart(mp.getBodyPart(i));
		} else if (o instanceof InputStream) {
			MyLog.info(this, "This is just an input stream");
			InputStream is = (InputStream) o;
			int c;
			while ((c = is.read()) != -1)
				System.out.write(c);
		}
	}

	//show briefing detail of all messages
	private void dumpEnvelope(Message m) throws Exception {
		Address[] a;
		if ((a = m.getFrom()) != null) {
			for (int j = 0; j < a.length; j++)
				MyLog.info(this, "FROM: " + a[j].toString());
			if ((a = m.getRecipients(Message.RecipientType.TO)) != null) {
				for (int j = 0; j < a.length; j++)
					MyLog.info(this, "TO: " + a[j].toString());
			}
			MyLog.info(this, "SUBJECT: " + m.getSubject());
			Date d = m.getSentDate();
			MyLog.info(this, "SendDate: " + (d != null ? d.toString() : "UNKNOWN"));
			Flags flags = m.getFlags();
			StringBuffer sb = new StringBuffer();
			Flags.Flag[] sf = flags.getSystemFlags(); // get the system flags
			boolean first = true;
			for (int i = 0; i < sf.length; i++) {
				String s;
				Flags.Flag f = sf[i];
				if (f == Flags.Flag.ANSWERED)
					s = "\\Answered";
				else if (f == Flags.Flag.DELETED)
					s = "\\Deleted";
				else if (f == Flags.Flag.DRAFT)
					s = "\\Draft";
				else if (f == Flags.Flag.FLAGGED)
					s = "\\Flagged";
				else if (f == Flags.Flag.RECENT)
					s = "\\Recent";
				else if (f == Flags.Flag.SEEN)
					s = "\\Seen";
				else
					continue; // skip it

				if (first)
					first = false;
				else
					sb.append(' ');
				sb.append(s);
			}
			String[] uf = flags.getUserFlags(); // get the user flag strings
			for (int i = 0; i < uf.length; i++) {
				if (first)
					first = false;
				else
					sb.append(' ');
				sb.append(uf[i]);
			}
			MyLog.info(this, "FLAGS = " + sb.toString());
		}
	}

	private int openMailBox(String name) throws Exception {
		this.folder = store.getFolder(name); //Open the Folder
		if (folder == null) {
			MyLog.info(this, "No default folder");
			return 0;
		}
		folder.open(Folder.READ_WRITE); //Open mailbox
		return 1;
	}

	private void closeMailBox() throws Exception {
		folder.close(false);
		store.close();
	}
}