package com.csc.library.report;

import java.io.InputStream;
import java.io.Reader;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.net.URL;
import java.rmi.RemoteException;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.Ref;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;

import com.csc.library.database.ChildManager;
import com.csc.library.database.ConnectionPools;
import com.csc.library.database.FieldDate;
import com.csc.library.database.FieldManager;
import com.csc.library.database.MyInquiry;
import com.csc.library.database.StaticProperties;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.DbRecord;
import com.csc.library.session.DbTable;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.DataLanguageXml;
import com.csc.library.utilities.DbXml;
import com.csc.library.utilities.FileString;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.UProfile;

public class ReportResultSet implements MyReport {
  private DbXml db;
  private DbInquiry di;
  private HashMap condition, orderByDesc;
  private ReportMetaData dm;
  private UProfile uProFile=null;
  private Properties fType;
  private boolean referLang=false;
  private String filter = "";
  private CscCalendar cscDate = new CscCalendar();
  private String Template="";
  private String processID = "";
  private int recCount = 0;
  private Vector reqFilter=new Vector(); 
  
  public UProfile getUProfile(){
  	return this.uProFile;
  }
  public ReportResultSet() {
    super();
  }

  protected void initField() {
    fType = new Properties();
    fType.setProperty("12", "String");
    fType.setProperty("8", "Double");
    fType.setProperty("4", "Int");
    fType.setProperty("91", "Date");
    fType.setProperty("92", "Time");
    fType.setProperty("93", "Timestamp");
  }

  private String getOrderByTag() {
    String orderby = "";
    String orderbyValue = "";
    for (int i = 1; i <= dm.getColumnOrderCount(); i++) {
      orderbyValue = (String) condition.get("ORDERBY" + i);
      if (orderbyValue != null && !orderbyValue.equals("")) {
        orderbyValue = orderbyValue.substring(orderbyValue.indexOf(".") + 1,
                                              orderbyValue.length());
        if (orderby.length()== 0)
          orderby += orderbyValue;
        else
          orderby += "," + orderbyValue;
      }
    }
    return orderby;
  }

  private Vector<String> getFullOrderByList(){
	  Vector<String> orderby=null;
	  try{
		if(this.getOrderByTag().length()>0){
			orderby = new Vector<String>(Arrays.asList((String[])this.getOrderByTag().toLowerCase().split(",")));
		}else{
			orderby = new Vector<String>();
		}
	    String[] orderByNew = ((String)this.condition.get("ORDERTEXT")).split(","); 
	    String temp;
	    for(int i=0;i<orderByNew.length;i++){
	      temp=orderByNew[i].toLowerCase();
	      if(temp.length()>0&&!(orderby.contains(temp)||orderby.contains(temp+" desc"))){
	          orderby.add(temp);
	      }
	    }
	    String[] orderdbxml=(String[])this.db.getInquiry().getOrderBy().split(",");
	    int maxsize=orderdbxml.length;
	    if(orderdbxml.length>3)
	    	maxsize=3;
	    for(int i=0;i<maxsize;i++){
		      temp=orderdbxml[i].toLowerCase();
		      if(temp.length()>0&&!(orderby.contains(temp)||orderby.contains(temp+" desc"))){
		          orderby.add(temp);
		      }
		    }
	    	    
	  }catch(Exception ex){
		  
	  }
	      return orderby;
	  }  
  

  private String getFullOrderBy(){
    String orderBy=this.getOrderByTag();
    String orderByNew = (String)this.condition.get("ORDERTEXT");
    String temp;
    java.util.StringTokenizer st=new java.util.StringTokenizer(orderByNew,",");
    while(st.hasMoreTokens()){
      temp=st.nextToken();
      if(orderBy.indexOf(temp)== -1){
              if(orderBy.length()==0)
                      orderBy+=temp;
              else
                orderBy=orderBy+","+temp;
      }
    }
    return orderBy;
  }

  /************nuball edit 29-11-06****************/
  private String getGroupBy(){
	  String temp="",groupByNew="";
	  String groupBy = (String)this.condition.get("GROUPTEXT");
	  if(groupBy ==null) {
		  groupByNew="";
	  } else {
		  java.util.StringTokenizer st=new java.util.StringTokenizer(groupBy," ");
		    while(st.hasMoreTokens()){
		      temp=st.nextToken().trim();
		      String str = temp.substring(temp.indexOf(".")+1, temp.length());
		      if(groupByNew.length()==0)
		    	  groupByNew+=str;
		      else
		    	  groupByNew=groupByNew+","+str;		     
		    }
	  }
	 
	  return groupByNew;
  }
  
  /*******************************************/
  private void setInquiry() {
    String dbXml = (String) condition.get("SCREEN");
    if(condition.containsKey("REPORT_XML"))
    	dbXml = (String) condition.get("REPORT_XML");
    MyLog.info(this, this.uProFile, "****************DBXML  "+dbXml);
    db = new DbXml(this.uProFile, dbXml, this.uProFile.getComCode(),
                   this.uProFile.getUserId(), this.uProFile.getPassword(),
                   this.condition);
    MyLog.info(this, this.uProFile, "****************DBXML  "+db);
    di = db.getInquiry();
    MyLog.info(this, this.uProFile, "****************INQUIRY  "+di);
    this.setInquiryCondition();
  }

  private void setInquiryCondition() {
	this.reqFilter=new Vector();
    Properties filter = (Properties) condition.get("FILTER");
    MyLog.info(this, this.uProFile, "----------------------- ReportResultSet Filter >" +filter + "<");
    try {
      if (filter != null) {
        String operator = (String) filter.get("operator");
//        this.setInquiryOperator(operator);
        this.setInquiryFilter(filter);
      }
      String order =getFullOrderByList().toString();
      order=order.replace("[","");
      order=order.replace("]","");
      MyLog.info(this, this.uProFile, "----------------------- ReportResultSet Order >" +order + "<");
      if (order.length()>0){
        di.setOrderBy(order);
      }
      
      /************nuball edit 29-11-06****************/
      MyLog.info(this, this.uProFile, "----------------------- ReportResultSet Group >" +this.getGroupBy() + "<");
      if((this.condition.get("GROUPDB" )!=null)&&(((String)this.condition.get("GROUPDB" )).equalsIgnoreCase("true"))&&(!this.getGroupBy().equalsIgnoreCase("")))
    	  di.setGroupBy(this.getGroupBy());
      /*******************************************/
      
      
      if (this.dm.getReferLang().equalsIgnoreCase("false"))
        this.referLang=false;
      else
        this.referLang=true;
      if(this.dm.getRequiredFound().equalsIgnoreCase("true")){
      	if(dm.getRequireFoundTable().length()>0)
      		di.setRequireFoundTable(dm.getRequireFoundTable());
      	else
      		di.setAllRequiredFound(true);
      } 
      //di.refresh();
    }
    catch (Exception e) {
    	 MyLog.error(this, this.uProFile, e);     
    }
  }

  private void setInquiryOperator(String operator) throws Exception {
  	 MyLog.info(this, this.uProFile, "Operator for inquiry " + operator);
    if (operator != null && operator.trim().equals("AND")) {
      di.setAllRequiredFound(true);
      MyLog.info(this, this.uProFile, "Set All RequireFound true");
    }
    else {
      di.setAllRequiredFound(false);
      MyLog.info(this, this.uProFile, "Set All RequireFound false");
    }
  }
  
  private void setInquiryFilter(Properties filter) throws Exception {
		DbInquiry inq = null;
		String tableName = di.getTableName();
		String fixpFilter = "", rFilter = "";
		int pre=-1,pos=-1;

		if (filter.get(tableName) != null|| this.condition.get("FIXFILTER" + tableName.toUpperCase()) != null) {
			fixpFilter = (String) this.condition.get("FIXFILTER"+ tableName.toUpperCase());
			if (fixpFilter != null && !fixpFilter.equals("")) {
				pre=fixpFilter.indexOf("$U{");
				pos=fixpFilter.indexOf("$F{");
				if(pre>-1 || pos>-1){
					fixpFilter=this.swapFormat(fixpFilter);
				}
				if(fixpFilter.indexOf("%25")>-1){
					fixpFilter=fixpFilter.replace("%25", "%");
				}
				if(fixpFilter.indexOf("%3C")>-1){
					fixpFilter=fixpFilter.replace("%3C", "<");
				}
				if(fixpFilter.indexOf("%3E")>-1){
					fixpFilter=fixpFilter.replace("%3E", ">");
				}

				rFilter += fixpFilter;
				if (filter.get(tableName) != null
						&& !filter.get(tableName).equals(""))
					rFilter += "  AND  " + (String) filter.get(tableName);
			} else {
				rFilter = (String) filter.get(tableName);
			}
			if (this.di.getType().equals("inquiry"))
				di.setFilter(rFilter);
			else 
				this.filter = rFilter;
		}
		this.setChildFilter(di.getChildTable(), filter,di);
		
		if(this.dm.getRequiredFound().equalsIgnoreCase("true")){
	      	if(reqFilter.size()>0){
	      		rFilter=di.getFilter();
	      		if(rFilter.length()>0)
	      			rFilter+=" and ";
	      		for(int i=0;i<reqFilter.size();i++){
	      			rFilter+=(String)reqFilter.get(i);
	      			if(i<reqFilter.size()-1)
	      				rFilter+=" and ";
	      		}
	      		di.setFilter(rFilter);
	      	 }
	      } 
	}

	private void setChildFilter(Hashtable h, Properties filter,DbInquiry inqParent) {
		try {
			ChildManager inq = null;
			Enumeration em = h.keys();
			String fixpFilter = "", rFilter = "", tableName = "",mainkey="";
			while (em.hasMoreElements()) {
				tableName = (String) em.nextElement();
				fixpFilter = "";
				rFilter = "";
				try {
					inq = inqParent.getChild(tableName);
					if (inq.getChildTable().size() > 0) {
						this.setChildFilter(inq.getChildTable(), filter,inqParent);
					}
				} catch (ClassCastException e) {
			    	 MyLog.error(this, this.uProFile, e );
			    	 MyLog.error(this, this.uProFile,"Object not instance Inquiry Can not set Filter");
				}
				if (filter.get(tableName) != null|| this.condition.get("FIXFILTER"
						+ tableName.toUpperCase()) != null) {
					fixpFilter = (String) this.condition.get("FIXFILTER"
							+ tableName.toUpperCase());
					if (fixpFilter != null && !fixpFilter.equals("")) {
						rFilter += fixpFilter;
						if (filter.get(tableName) != null && 
								!filter.get(tableName).equals(""))
							rFilter += "  AND  " + (String) filter.get(tableName);
					} else
						rFilter = (String) filter.get(tableName);
					inq.setFilter(rFilter);
				   if(di.getRelation(inq.getActualName())!=null){
					if(inq instanceof DbRecord){
						mainkey=((DbRecord)inq).getMainkey().toLowerCase();
					}else{
						mainkey=((DbInquiry)inq).getCurrentDbRecord().getMainkey().toLowerCase();
					}
					if(!di.getCurrentDbRecord().getArrayFieldList().contains(mainkey)){
						mainkey=di.getCurrentDbRecord().getMainkey().toLowerCase();
					}
					reqFilter.add(mainkey+" in (select "+mainkey+" from "+inq.getTableName()+" where "+rFilter+")");
				   }
					
					// reform filter for any object
					if (this.di.getType().equals("inqreport")) {
						String actualName = inq.getActualName();
						String tempFilter = rFilter;
						String fullFilter = "";
						while (tempFilter.length() > 0) {
							int andPos = tempFilter.indexOf("and");
							int orPos = tempFilter.indexOf("or");
							if ((andPos < orPos || orPos == -1) && andPos != -1) {
								String subSql = tempFilter.substring(0, andPos).trim();
								subSql = actualName + "." + subSql;
								tempFilter = tempFilter.substring(andPos + 3).trim();
								fullFilter = fullFilter + subSql + " and ";
							} else if ((orPos < andPos || andPos ==-1) && orPos != -1 ) { 
								String subSql = tempFilter.substring(0, orPos).trim();
								subSql = actualName + "." + subSql;
								tempFilter = tempFilter.substring(orPos + 2).trim();
								fullFilter = fullFilter + subSql + " or ";
							} else {
								fullFilter = fullFilter + actualName + "." + tempFilter;
								tempFilter = "";
							}
						}
						rFilter = fullFilter;
					}
					if (this.filter != null && !this.filter.equals("")) {
						this.filter = this.filter + " and " + rFilter;
					} else {
						this.filter = rFilter;
					}
				}
			}
			if (this.di.getType().equals("inqreport"))
				di.setFilter(this.filter);
		} catch (RemoteException e) {
			MyLog.error(this, this.uProFile, e);
		}
	}

  public void setMetaData(ReportMetaData dm) {
    this.dm = dm;
    dm.setUProfile(this.uProFile);
  }

  private void setMetaData() {
//    System.out.println("Report Metadata *********************************************");
    dm = new ReportMetaData();
    dm.setDbName(this.uProFile.getDbName());
//     System.out.println("Report Metadata ddddddddddddd*********************************************");
    dm.setUProfile(this.uProFile); 
    dm.setReportXml( (String) condition.get("SCREEN"));
    try {
      dm.setFieldObject(this.Template);	
      dm.process();
    }
    catch (Exception ex) {
    	MyLog.error(this, this.uProFile, ex);
    }
  }

  private String getChildData(String columnName, String tableName) {
    ChildManager diChild = null;
		// ChildManager objCh = null;
		String data = "";
		try {
			diChild = (ChildManager) di.getChild(tableName);
			if (diChild != null) {
				if (diChild instanceof MyInquiry) {
					DbInquiry di = (DbInquiry) diChild;
					FieldManager fm = di.getCurrentDbRecord().getField(columnName);
					if(fm!=null){
						if (fm instanceof FieldDate) {
							data = fm.getDataFormat();
						} else {
							data = fm.getString();
						}
					}else{
						data = di.getString(columnName);
					}
				} else {//child is record
					data = diChild.getString(columnName);
				}
			}else{
	    		MyLog.debug(this, di.getUProfile(), "Could not find table : "+tableName);
			}
		} catch (RemoteException ex) {
			MyLog.error(this, this.uProFile, ex);
		}
		return data;
	}

  private String getParentData(String columnName) {
    try {
    	FieldManager fm = di.getCurrentDbRecord().getField(columnName);
    	if(fm!=null){
	    	if (fm instanceof FieldDate) {
	    		return fm.getDataFormat();
	    	}
	    	return fm.getString();
	   	}else{
	   		return di.getString(columnName);
	   	}
    }
    catch (RemoteException ex) {
    	MyLog.error(this, this.uProFile,	ex);
      return null;
    }
  }

  private String getOrderBy(String columnName) {
    try {
      String fullColumnName = columnName;
      String data = "";
      if (condition.get(columnName.toUpperCase()) != null) {
        columnName = (String) condition.get(columnName.toUpperCase());
        String tableName=columnName.substring(0,columnName.indexOf("."));
        columnName = columnName.substring(columnName.indexOf(".") + 1,
                                          columnName.length());
        if(di.getTableName().equalsIgnoreCase(tableName)){
      		data=di.getString(columnName);
      	}else{
      		data = di.getString(tableName,columnName);
      	}
      }else if(columnName.indexOf("desc")!=-1){
      	columnName=(String)this.dm.getFieldOrderDesc().get(columnName.toUpperCase());
        if (columnName==null) return "";
      	String tableName=columnName.substring(0,columnName.indexOf("@"));
      	//columnName=columnName.substring(columnName.indexOf("@"+1,columnName.length()));      	
      	columnName=columnName.substring(columnName.indexOf("@")+1,columnName.length());
      	if(di.getTableName().equalsIgnoreCase(tableName)){
      		data=di.getString(columnName);
      	}else{
      		data = di.getString(tableName,columnName);
      	}
      }
	  if (data==null) {
	   	data = this.getDefault(fullColumnName);
	  }
      return data;
    }
    catch (Exception ex) {
    	MyLog.error(this, this.uProFile,	ex);
    	return "";
    }
  }

  private void setOrderByDesc() {
    String value = "";
    for (int i = 1; i <= dm.getColumnOrderCount(); i++) {
      if (condition.containsKey("ORDERBY" + i)) {
        value = (String) condition.get("ORDERBY" + i);
        dm.setFieldOrderDesc("ORDERBY" + i + "DESC", value);
        dm.setFieldOrderLabel(value);
      }
    }
  }

  private String getFieldDB(String tableName, String columnName) {
    // String parentTable = "";
    try {
  	  	String data = "";
	    if(columnName.indexOf("9_9")!=-1)  {
	      	FileString fs=new FileString();
	    	fs.setData(columnName);
	    	columnName=fs.replaceAll("9_9","$");
	    }      
	   	String fullColumnName = columnName;
        // parentTable =
		// di.getCurrentDbRecord().getClass().getName().toLowerCase();
      	
      	
      // parentTable = di.getTableName();
        if (tableName.equalsIgnoreCase(di.getTableName())||tableName.equalsIgnoreCase(di.getActualName())) { 
          data= this.getParentData(columnName);
        } else {
          data= this.getChildData(columnName, tableName);
        }
        if (data==null) {
        	data = this.getDefault(fullColumnName);
        }
        return data;
      }
      catch (Exception ex) { 
    	MyLog.error(this, this.uProFile,	ex);
      	return null;
      }
  }


/*  private String getFieldDB(String tableName, String columnName) {
    String parentTable = "";
    try {
      //parentTable = di.getCurrentDbRecord().getClass().getName().toLowerCase();
      parentTable = di.getTableName();
      if (parentTable.indexOf(tableName.toLowerCase()) != -1)
        return this.getParentData(columnName);
      else
        return this.getChildData(columnName, tableName);
    }
    catch (Exception ex) {
      return null;
    }
  }
*/
  
  private String getDefault(String fullColumnName) {
  	String returnValue = "";
		int type =  dm.getColumnType(fullColumnName);
	  	returnValue = "";
	  	
	  	if (type==8 || type==4) {
	  		returnValue = "0";
	  	} else {
		if (type==91) {
			Date d = new Date(0);
			returnValue = d.toString();
	  	} else {
		if (type==92) {
			Time t = new Time(0);
			returnValue = t.toString();
	   	} else {
	   	if (type==93) {
			Timestamp ts = new Timestamp(0);
			returnValue = ts.toString();
	   	}
	   	}
	  	}
	  	}
	  	return returnValue;
  }
  
  private String getFieldData(String columnName) {
    String tableName, parentTable, returnValue = "";
    columnName=columnName.toLowerCase();
    String fullColumnName = columnName;
    try {
      if (this.referLang) {
        this.di.referLangOn();
      }
      if(columnName.indexOf("isfmchg")==0){
      	returnValue = "0";
      		try {
				if(this.getBoolean(columnName.substring("isfmchg".length()))) 
						returnValue = "1";
			} catch (SQLException e) {
		    	MyLog.error(this, this.uProFile,	e);
			}
      }else
      if (dm.getDbFieldList().contains(columnName)) {
        tableName = columnName.substring(0, columnName.indexOf(dm.getSign()));
        columnName = columnName.substring(columnName.indexOf(dm.getSign()) + 2,columnName.length());
        returnValue = this.getFieldDB(tableName, columnName);
      }
      else {
        returnValue = this.getOrderBy(columnName);
      }
      if (returnValue==null) {
      	returnValue = this.getDefault(fullColumnName);
      }
      di.referLangOff();
    }
    catch (RemoteException ex) {
    	MyLog.error(this, this.uProFile,	ex);
    }
    return returnValue;
  }

  private String getColumnName(int columnIndex) {
    try {
      return dm.getColumnName(columnIndex);
    }
    catch (SQLException ex) {
    	MyLog.error(this, this.uProFile,	ex);
    }
    return null;
  }

  private Integer getIntObject(Integer columnIndex) {
          try {
                return new Integer(this.getFieldData(this.getColumnName(columnIndex.intValue())));
          } catch (RuntimeException e) {
          		MyLog.error(this, this.uProFile,	"Error IN getIntObject : " + this.getColumnName(columnIndex.intValue()));
                MyLog.error(this, this.uProFile,	e);
          }
          return new Integer(0);
  }

  private Boolean getBooleanObject(Integer columnIndex) {
          return new Boolean(this.getFieldData(this.getColumnName(columnIndex.intValue())));
          // return true;
  }

  private Double getDoubleObject(Integer columnIndex) {
          return new Double(this.getFieldData(this.getColumnName(columnIndex.intValue())));
  }

  public String getStringObject(Integer columnIndex) throws java.sql.SQLException {
          return this.getFieldData(this.getColumnName(columnIndex.intValue()));
  }

  public Timestamp getTimestampObject(Integer columnIndex) throws java.sql.SQLException {
          return Timestamp.valueOf(this.getFieldData(this.getColumnName(columnIndex.intValue())));
  }
  public Date getDateObject(Integer columnIndex) throws java.sql.SQLException {
          return Date.valueOf(this.getFieldData(this.getColumnName(columnIndex.intValue())));
  }
  public Time getTimeObject(Integer columnIndex) throws java.sql.SQLException {
          return Time.valueOf(this.getFieldData(this.getColumnName(columnIndex.intValue())));
  }
  public BigDecimal getBigDecimalObject(Integer columnIndex) throws java.sql.SQLException {
          return BigDecimal.valueOf(Long.valueOf(this.getFieldData(this.getColumnName(columnIndex.intValue()))).longValue());
  }

  public Object getObject(int columnIndex) {
          try {
                String type = this.fType.getProperty(String.valueOf(dm.getColumnType(columnIndex)));
                Method m = this.getClass().getDeclaredMethod("get" + type + "Object", new Class[] { java.lang.Integer.class });
                return m.invoke(this, new Object[] { new Integer(columnIndex)});
          } catch (Exception ex) {
          		MyLog.error(this, this.uProFile,	ex);
                return null;
          }
  }

  private boolean next(String chName){
      try {
      		if (StaticProperties.serverManager.isCancle(this.processID)) {
      			return false;
      		}
            if(di.recNo()==0){
                return di.next();
            }
          boolean chnext=false;
          Object objChild=di.getChild(chName);
          if(objChild instanceof DbRecord || objChild instanceof DbTable){
        	  return false;
          }else{
	          DbInquiry ch=(DbInquiry)objChild;
	          try {
	                  chnext=ch.next();
	          } catch (Exception e) {
	        	  	  MyLog.error(this, this.uProFile,	"Can't next Record Child :"+e);
	                  chnext=false;
	          }
	          if(!chnext){
	        	  if(di.recNo()==this.getRow())
	        		  return false;
	              else
	            	  return di.next();
	          }else
	              return true;
          }
      } catch (Exception e) {
      		MyLog.error(this, this.uProFile,	e);
            return false;
      }
}



  public boolean next()  {
		boolean ok = false;
        try {
	      		if (StaticProperties.serverManager.isCancle(this.processID)) {
	      			return false;
	      		}
              String childChk=(String)this.condition.get("CHILDCHK");
              if(childChk !=null && !childChk.equals("")) {
                   ok=this.next(childChk);
              } else {
                   ok=di.next();
              }
              if (ok) {
              	this.recCount++;
              	StaticProperties.serverManager.aliveProcess(this.processID, "Printing record number : "+this.recCount);	
              }
              return ok;
        } catch (Exception ex) {
        		MyLog.error(this, this.uProFile,	ex);
              return false;
        }
}

  /**
   * Pas 28122005
   *  ödatatadic ͧŵͧ
   *  UProfile 仴
   *   @see com.csc.library.report.MyReport#getThaiLabel(java.lang.String, java.lang.String)
   **/
  public String getThaiLabel(String columnName, String dataIndex) {
    String label = "";
    //label = new DataLanguageXml(this.uProFile.getDbName(), columnName,
   //                             dataIndex).getFieldLabel();
    label = new DataLanguageXml(this.uProFile, columnName,dataIndex).getFieldLabel();    
    return label;
  }

  public void process(HashMap condition) {
    this.condition = condition;
    this.Template=this.condition.get("TEMPLATE").toString();
    this.uProFile = (UProfile)this.condition.get("UPROFILE");
    if (this.uProFile.containsKey("processID")) {
    	this.processID = this.uProFile.get("processID").toString();
  	}
    this.setMetaData();
    //System.out.println("Field RS "+dm.getCollumnName());
    this.setInquiry();
    try{
      di.refresh();
    }catch(Exception e){e.printStackTrace();}
    this.initField();
    this.setOrderByDesc();
    this.setUprofileLang();
  }

  private void setUprofileLang() {
    if (this.condition.get("LANGUAGE") != null && this.condition.get("LANGUAGE").equals("ENG"))
      this.uProFile.set("lang", "ENG");
    else
      this.uProFile.set("lang", "THA");
  }

  public String getString(int columnIndex) throws java.sql.SQLException {
    return "";
  }

  public String getString(String columnName) {
       return this.getFieldData(columnName);
  }

 

  public void close() throws java.sql.SQLException {
  }

  public ResultSetMetaData getMetaData() {
    return dm;
  }

  public ReportMetaData getReportMetaData() {
    return dm;
  }

  public boolean wasNull() {
    return false;
  }

//-- ResultSet Method --------------------------------------------------------//
  public Object getObject(String columnName) {
	  		columnName=columnName.toLowerCase();
  			String s = this.getFieldData(columnName);
  			int type =  dm.getColumnType(columnName);
/*  			
  		    fType.setProperty("12", "String");
  		    fType.setProperty("8", "Double");
  		    fType.setProperty("4", "Int");
  		    fType.setProperty("91", "Date");
  		    fType.setProperty("92", "Time");
  		    fType.setProperty("93", "Timestamp");
*/  		  	
  		  	if (type==8) {
  				if (s==null || s.trim().equals("")) {
  					s = "0";
  				}
  	  			if(!dm.isFieldObject()){
  	  				return s;
  	  			} else {
  	  		  		Double d = new Double(s);
  	  		  		return d;
  	  			}
 		  	} else {
  			if (type==4) {
  				if (s==null || s.trim().equals("")) {
  					s = "0";
  				}
  	  			if(!dm.isFieldObject()){
  	  				return s;
  	  			} else {
  	  				Integer i = new Integer(s);
  	  				return i;
  	  			}
  		  	}else {
	  		if (type==91) {
  	  			if(!dm.isFieldObject()){
  	  				return s;
  	  			} else {
	  				cscDate.setCscCalendar(s);
	  				Date d;
	  				d = Date.valueOf(cscDate.getYYYYMMDD());
	  				return d;
  	  			}
  		  	} else {
  			if (type==92) {
  	  			if(!dm.isFieldObject()){
  	  				return s;
  	  			} else {
	  				cscDate.setCscCalendar(s);
	  				Time t;
	  				t=Time.valueOf(cscDate.getHHMMSS());
	  				return t;
  	  			}
  		   	} else {
  		   	if (type==93) {
  	  			if(!dm.isFieldObject()){
  	  				return s;
  	  			} else {
	  				cscDate.setCscCalendar(s);
	  				Timestamp ts;
	  				ts=Timestamp.valueOf(cscDate.getYYYYMMDDHHMMSS());
	  				return ts;
  	  			}
  		   	}
  		   	}
  		  	}
  		  	}
 		  	}
  		  	return s;
   }
  
 public boolean getBoolean(int columnIndex) {
         return true;
 }
 public byte getByte(int columnIndex) {
         return 0;
 }
 public short getShort(int columnIndex) {
         return 0;
 }
 public int getInt(int columnIndex) {
         return 0;
 }
 public long getLong(int columnIndex) {
         return 0;
 }
 /**
  * Because the address file does not have the data in this type, leave the method empty.
  */
 public float getFloat(int columnIndex) {
         return 0;
 }
 /**
  * Because the address file does not have data in this type, leave the method empty.
  */
 public double getDouble(int columnIndex) {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#next()
  */
 /**
  * @see java.sql.ResultSet#getString(int)
  */
 /**
  * @see java.sql.ResultSet#getInt(int)
  */
 /**
  * @see java.sql.ResultSet#getDouble(int)
  */
 /**
  * @see java.sql.ResultSet#getBigDecimal(int, int)
  * @deprecated
  */
 public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBytes(int)
  */
 public byte[] getBytes(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getDate(int)
  */
 public Date getDate(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTime(int)
  */
 public Time getTime(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTimestamp(int)
  */
 public Timestamp getTimestamp(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getAsciiStream(int)
  */
 public InputStream getAsciiStream(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getUnicodeStream(int)
  * @deprecated
  */
 public InputStream getUnicodeStream(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBinaryStream(int)
  */
 public InputStream getBinaryStream(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getString(String)
  */
 /**
  * @see java.sql.ResultSet#getBoolean(String)
  */
 public boolean getBoolean(String columnName) throws SQLException {
 	String tableName = columnName.substring(0,columnName.indexOf("__"));
 	String fieldName = columnName.substring(columnName.indexOf("__")+2); 
 	return this.isFieldChange(tableName,fieldName);
 }
 /**
  * @see java.sql.ResultSet#getByte(String)
  */
 public byte getByte(String columnName) throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getShort(String)
  */
 public short getShort(String columnName) throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getInt(String)
  */
 public int getInt(String columnName) throws SQLException {
         return Integer.parseInt(this.getFieldData(columnName));
 }
 /**
  * @see java.sql.ResultSet#getLong(String)
  */
 public long getLong(String columnName) throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getFloat(String)
  */
 public float getFloat(String columnName) throws SQLException {
         return Float.parseFloat(this.getFieldData(columnName));
 }
 /**
  * @see java.sql.ResultSet#getDouble(String)
  */
 public double getDouble(String columnName) throws SQLException {
         return Double.parseDouble(columnName);
 }
 /**
  * @see java.sql.ResultSet#getBigDecimal(String, int)
  * @deprecated
  */
 public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBytes(String)
  */
 public byte[] getBytes(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getDate(String)
  */
 public Date getDate(String columnName) throws SQLException {
         return Date.valueOf(this.getFieldData(columnName));
 }
 /**
  * @see java.sql.ResultSet#getTime(String)
  */
 public Time getTime(String columnName) throws SQLException {
         return Time.valueOf(this.getFieldData(columnName));
 }
 /**
  * @see java.sql.ResultSet#getTimestamp(String)
  */
 public Timestamp getTimestamp(String columnName) throws SQLException {
         return Timestamp.valueOf(this.getFieldData(columnName));
 }
 /**
  * @see java.sql.ResultSet#getAsciiStream(String)
  */
 public InputStream getAsciiStream(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getUnicodeStream(String)
  * @deprecated
  */
 public InputStream getUnicodeStream(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBinaryStream(String)
  */
 public InputStream getBinaryStream(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getWarnings()
  */
 public SQLWarning getWarnings() throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#clearWarnings()
  */
 public void clearWarnings() throws SQLException {}
 /**
  * @see java.sql.ResultSet#getCursorName()
  */
 public String getCursorName() throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getMetaData()
  */
 /**
  * @see java.sql.ResultSet#getObject(String)
  */

 /**
  * @see java.sql.ResultSet#findColumn(String)
  */
 public int findColumn(String columnName) throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getCharacterStream(int)
  */
 public Reader getCharacterStream(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getCharacterStream(String)
  */
 public Reader getCharacterStream(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBigDecimal(int)
  */
 public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBigDecimal(String)
  */
 public BigDecimal getBigDecimal(String columnName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#isBeforeFirst()
  */
 public boolean isBeforeFirst() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#isAfterLast()
  */
 public boolean isAfterLast() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#isFirst()
  */
 public boolean isFirst() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#isLast()
  */
 public boolean isLast() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#beforeFirst()
  */
 public void beforeFirst() throws SQLException {}
 /**
  * @see java.sql.ResultSet#afterLast()
  */
 public void afterLast() throws SQLException {}
 /**
  * @see java.sql.ResultSet#first()
  */
 public boolean first() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#last()
  */
 public boolean last() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#getRow()
  */
 public int getRow() throws SQLException {
        try {
          return this.di.recCount();
        }
        catch (RemoteException ex) {
        	MyLog.error(this, this.uProFile,	ex);
          return 0;
        }
 }
 /**
  * @see java.sql.ResultSet#absolute(int)
  */
 public boolean absolute(int row) throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#relative(int)
  */
 public boolean relative(int rows) throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#previous()
  */
 public boolean previous() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#setFetchDirection(int)
  */
 public void setFetchDirection(int direction) throws SQLException {}
 /**
  * @see java.sql.ResultSet#getFetchDirection()
  */
 public int getFetchDirection() throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#setFetchSize(int)
  */
 public void setFetchSize(int rows) throws SQLException {}
 /**
  * @see java.sql.ResultSet#getFetchSize()
  */
 public int getFetchSize() throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getType()
  */
 public int getType() throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#getConcurrency()
  */
 public int getConcurrency() throws SQLException {
         return 0;
 }
 /**
  * @see java.sql.ResultSet#rowUpdated()
  */
 public boolean rowUpdated() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#rowInserted()
  */
 public boolean rowInserted() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#rowDeleted()
  */
 public boolean rowDeleted() throws SQLException {
         return false;
 }
 /**
  * @see java.sql.ResultSet#updateNull(int)
  */
 public void updateNull(int columnIndex) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBoolean(int, boolean)
  */
 public void updateBoolean(int columnIndex, boolean x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateByte(int, byte)
  */
 public void updateByte(int columnIndex, byte x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateShort(int, short)
  */
 public void updateShort(int columnIndex, short x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateInt(int, int)
  */
 public void updateInt(int columnIndex, int x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateLong(int, long)
  */
 public void updateLong(int columnIndex, long x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateFloat(int, float)
  */
 public void updateFloat(int columnIndex, float x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateDouble(int, double)
  */
 public void updateDouble(int columnIndex, double x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBigDecimal(int, BigDecimal)
  */
 public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateString(int, String)
  */
 public void updateString(int columnIndex, String x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBytes(int, byte[])
  */
 public void updateBytes(int columnIndex, byte[] x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateDate(int, Date)
  */
 public void updateDate(int columnIndex, Date x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateTime(int, Time)
  */
 public void updateTime(int columnIndex, Time x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateTimestamp(int, Timestamp)
  */
 public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateAsciiStream(int, InputStream, int)
  */
 public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBinaryStream(int, InputStream, int)
  */
 public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateCharacterStream(int, Reader, int)
  */
 public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateObject(int, Object, int)
  */
 public void updateObject(int columnIndex, Object x, int scale) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateObject(int, Object)
  */
 public void updateObject(int columnIndex, Object x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateNull(String)
  */
 public void updateNull(String columnName) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBoolean(String, boolean)
  */
 public void updateBoolean(String columnName, boolean x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateByte(String, byte)
  */
 public void updateByte(String columnName, byte x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateShort(String, short)
  */
 public void updateShort(String columnName, short x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateInt(String, int)
  */
 public void updateInt(String columnName, int x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateLong(String, long)
  */
 public void updateLong(String columnName, long x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateFloat(String, float)
  */
 public void updateFloat(String columnName, float x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateDouble(String, double)
  */
 public void updateDouble(String columnName, double x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBigDecimal(String, BigDecimal)
  */
 public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateString(String, String)
  */
 public void updateString(String columnName, String x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBytes(String, byte[])
  */
 public void updateBytes(String columnName, byte[] x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateDate(String, Date)
  */
 public void updateDate(String columnName, Date x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateTime(String, Time)
  */
 public void updateTime(String columnName, Time x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateTimestamp(String, Timestamp)
  */
 public void updateTimestamp(String columnName, Timestamp x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateAsciiStream(String, InputStream, int)
  */
 public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateBinaryStream(String, InputStream, int)
  */
 public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateCharacterStream(String, Reader, int)
  */
 public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateObject(String, Object, int)
  */
 public void updateObject(String columnName, Object x, int scale) throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateObject(String, Object)
  */
 public void updateObject(String columnName, Object x) throws SQLException {}
 /**
  * @see java.sql.ResultSet#insertRow()
  */
 public void insertRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#updateRow()
  */
 public void updateRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#deleteRow()
  */
 public void deleteRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#refreshRow()
  */
 public void refreshRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#cancelRowUpdates()
  */
 public void cancelRowUpdates() throws SQLException {}
 /**
  * @see java.sql.ResultSet#moveToInsertRow()
  */
 public void moveToInsertRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#moveToCurrentRow()
  */
 public void moveToCurrentRow() throws SQLException {}
 /**
  * @see java.sql.ResultSet#getStatement()
  */
 public Statement getStatement() throws SQLException {
         return null;
 }

 /**
  * @see java.sql.ResultSet#getRef(int)
  */
 public Ref getRef(int i) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBlob(int)
  */
 public Blob getBlob(int i) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getClob(int)
  */
 public Clob getClob(int i) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getArray(int)
  */
 public Array getArray(int i) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getRef(String)
  */
 public Ref getRef(String colName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getBlob(String)
  */
 public Blob getBlob(String colName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getClob(String)
  */
 public Clob getClob(String colName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getArray(String)
  */
 public Array getArray(String colName) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getDate(int, Calendar)
  */
 public Date getDate(int columnIndex, Calendar cal) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getDate(String, Calendar)
  */
 public Date getDate(String columnName, Calendar cal) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTime(int, Calendar)
  */
 public Time getTime(int columnIndex, Calendar cal) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTime(String, Calendar)
  */
 public Time getTime(String columnName, Calendar cal) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTimestamp(int, Calendar)
  */
 public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
         return null;
 }
 /**
  * @see java.sql.ResultSet#getTimestamp(String, Calendar)
  */
 public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException {
         return null;
 }
 //For 1.4
 public void updateClob(int columnName, Clob cal) throws SQLException {}
 public void updateClob(String columnName, Clob cal) throws SQLException {}
 public void updateArray(int columnName, Array cal) throws SQLException {}
 public void updateArray(String columnName, Array cal) throws SQLException {}
 public void updateBlob(int columnName, Blob cal) throws SQLException {}
 public void updateBlob(String columnName, Blob cal) throws SQLException {}
 public void updateRef(int columnName, Ref cal) throws SQLException {}
 public void updateRef(String columnName, Ref cal) throws SQLException {}
 public URL getURL(int columnName) throws SQLException {
         return null;
 }
 public URL getURL(String columnName) throws SQLException {
         return null;
 }
	private boolean isFieldChange(String tableName,String fieldName) {
		try {
			if (di.getTableName().equals(tableName)) {
				return di.isFieldChange(fieldName);
			} else {
				return ((DbInquiry) di.getChild(tableName)).isFieldChange(fieldName);
			}
		} catch (RemoteException e) {
	    	MyLog.error(this, this.uProFile,	e);
		}
		return true;
	}
	private String swapFormat(String data){
		/***
		 * $U{keyname}  keyname in UProfile
		 * in ReportXML ==> companyid='$U%7Bcompanyid%7D' 
		 */
		String rp=data.substring(data.indexOf("$"),data.indexOf("}")+1);		
		String fieldName=rp.substring(3,rp.length()-1);
		if(data.indexOf("$U")>-1){
			try {
				data=data.replace(rp,this.getUProfile().get(fieldName).toString());
			} catch (Exception e) {
				data=data.replace(rp,"");
			}
		}
		if(data.indexOf("$U")>-1)
			data=swapFormat(data);

		return data;
	}
	public int getHoldability() throws SQLException {
		return 0;
	}
	public Reader getNCharacterStream(int columnIndex) throws SQLException {
		return null;
	}
	public Reader getNCharacterStream(String columnLabel) throws SQLException {
		return null;
	}
//	public NClob getNClob(int columnIndex) throws SQLException {
//		return null;
//	}
//	public NClob getNClob(String columnLabel) throws SQLException {
//		return null;
//	}
	public String getNString(int columnIndex) throws SQLException {
		return null;
	}
	public String getNString(String columnLabel) throws SQLException {
		return null;
	}

	public Object getObject(String columnLabel, Map<String, Class<?>> map)
			throws SQLException {
		return null;
	}
//	public RowId getRowId(int columnIndex) throws SQLException {
//		return null;
//	}
//	public RowId getRowId(String columnLabel) throws SQLException {
//		return null;
//	}
//	public SQLXML getSQLXML(int columnIndex) throws SQLException {
//		return null;
//	}
//	public SQLXML getSQLXML(String columnLabel) throws SQLException {
//		return null;
//	}
	public boolean isClosed() throws SQLException {
		return false;
	}
	public void updateAsciiStream(int columnIndex, InputStream x)
			throws SQLException {	
	}
	public void updateAsciiStream(String columnLabel, InputStream x)
			throws SQLException {	
	}
	public void updateAsciiStream(int columnIndex, InputStream x, long length)
			throws SQLException {
		
	}
	public void updateAsciiStream(String columnLabel, InputStream x, long length)
			throws SQLException {		
	}
	public void updateBinaryStream(int columnIndex, InputStream x)
			throws SQLException {	
	}
	public void updateBinaryStream(String columnLabel, InputStream x)
			throws SQLException {
	}
	public void updateBinaryStream(int columnIndex, InputStream x, long length)
			throws SQLException {		
	}
	public void updateBinaryStream(String columnLabel, InputStream x,
			long length) throws SQLException {		
	}
	public void updateBlob(int columnIndex, InputStream inputStream)
			throws SQLException {	
	}
	public void updateBlob(String columnLabel, InputStream inputStream)
			throws SQLException {		
	}
	public void updateBlob(int columnIndex, InputStream inputStream, long length)
			throws SQLException {
	}
	public void updateBlob(String columnLabel, InputStream inputStream,
			long length) throws SQLException {		
	}
	public void updateCharacterStream(int columnIndex, Reader x)
			throws SQLException {	
	}
	public void updateCharacterStream(String columnLabel, Reader reader)
			throws SQLException {
	}
	public void updateCharacterStream(int columnIndex, Reader x, long length)
			throws SQLException {		
	}
	public void updateCharacterStream(String columnLabel, Reader reader,
			long length) throws SQLException {		
	}
	public void updateClob(int columnIndex, Reader reader) throws SQLException {		
	}
	public void updateClob(String columnLabel, Reader reader)
			throws SQLException {		
	}
	public void updateClob(int columnIndex, Reader reader, long length)
			throws SQLException {		
	}
	public void updateClob(String columnLabel, Reader reader, long length)
			throws SQLException {		
	}
	public void updateNCharacterStream(int columnIndex, Reader x)
			throws SQLException {		
	}
	public void updateNCharacterStream(String columnLabel, Reader reader)
			throws SQLException {		
	}
	public void updateNCharacterStream(int columnIndex, Reader x, long length)
			throws SQLException {		
	}
	public void updateNCharacterStream(String columnLabel, Reader reader,
			long length) throws SQLException {		
	}
//	public void updateNClob(int columnIndex, NClob clob) throws SQLException {
//		
//	}
//	public void updateNClob(String columnLabel, NClob clob) throws SQLException {
//		
//	}
	public void updateNClob(int columnIndex, Reader reader) throws SQLException {		
	}
	public void updateNClob(String columnLabel, Reader reader)
			throws SQLException {		
	}
	public void updateNClob(int columnIndex, Reader reader, long length)
			throws SQLException {		
	}
	public void updateNClob(String columnLabel, Reader reader, long length)
			throws SQLException {		
	}
	public void updateNString(int columnIndex, String string)
			throws SQLException {		
	}
	public void updateNString(String columnLabel, String string)
			throws SQLException {
	}
//	public void updateRowId(int columnIndex, RowId x) throws SQLException {
//	}
//	public void updateRowId(String columnLabel, RowId x) throws SQLException {
//	}
//	public void updateSQLXML(int columnIndex, SQLXML xmlObject)
//			throws SQLException {
//	}
//	public void updateSQLXML(String columnLabel, SQLXML xmlObject)
//			throws SQLException {		
//	}
	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return false;
	}
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return null;
	}
	public Object getObject(int columnIndex, Map<String, Class<?>> map)
			throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public RowId getRowId(int columnIndex) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public RowId getRowId(String columnLabel) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public void updateRowId(int columnIndex, RowId x) throws SQLException {
		// TODO Auto-generated method stub
		
	}
	public void updateRowId(String columnLabel, RowId x) throws SQLException {
		// TODO Auto-generated method stub
		
	}
	public void updateNClob(int columnIndex, NClob nClob) throws SQLException {
		// TODO Auto-generated method stub
		
	}
	public void updateNClob(String columnLabel, NClob nClob)
			throws SQLException {
		// TODO Auto-generated method stub
		
	}
	public NClob getNClob(int columnIndex) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public NClob getNClob(String columnLabel) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public SQLXML getSQLXML(int columnIndex) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public SQLXML getSQLXML(String columnLabel) throws SQLException {
		// TODO Auto-generated method stub
		return null;
	}
	public void updateSQLXML(int columnIndex, SQLXML xmlObject)
			throws SQLException {
		// TODO Auto-generated method stub
		
	}
	public void updateSQLXML(String columnLabel, SQLXML xmlObject)
			throws SQLException {
		// TODO Auto-generated method stub
		
	}	

	/*  genREport ᷹
	@Override
	protected void finalize(){
	try{
		if(this.processID!=null &&this.processID.length()>0){
			ConnectionPools cp = (ConnectionPools) StaticProperties.connectionMap.get((String)this.getUProfile().getDbName());
			cp.removeConnectionProcess(this.processID);
			this.processID="";
		}
	}catch(Exception ex){
		MyLog.error(this, ex);
	}
		
	}
	*/
}
