package com.csc.library.report;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Properties;

import com.csc.library.utilities.MyLog;

public class TemplateFormula {
	private HashMap condition;
	private HashMap data, orderByDesc;
	private MyReport myrep;
	private Properties formulaMethod;

	public TemplateFormula(HashMap condition, MyReport myrep) {
		this.condition = condition;
		this.myrep = myrep;
		this.setMethodProperties();
	}

	private void setMethodProperties() {
		formulaMethod = new Properties();
		formulaMethod.setProperty("PAGENUMBER", "getPageNumberFormula");
//		formulaMethod.setProperty("TOTALPAGENUMBER", "getTotalPageCountFormula");
		formulaMethod.setProperty("ONNEWPAGE", "getOnnewPageFormula");
		formulaMethod.setProperty("RECCOUNT", "getRecCountFormula");
		formulaMethod.setProperty("PSTIME", "getBeforPrintFormula");
		formulaMethod.setProperty("PETIME", "getAffterPrintFormula");
		if(myrep!=null){
		for (int i = 1; i <= myrep.getReportMetaData().getColumnOrderCount(); i++) {
			//formulaMethod.setProperty("INVISIBLE_VIRTUALFIELD.ORDERBY" + i, "getInvisibleFormular");
			formulaMethod.setProperty("INVISIBLE_ORDERBY" + i, "getInvisibleFormular");
			formulaMethod.setProperty("NEWPAGEORDER" + i, "getNewPageGroupFormular");
			formulaMethod.setProperty("ORDERBY" + i + "DESC", "getOrderByDescFormula");
		}
	  }
	}

	public String getPromptThaiLabel(String name, String value) {
		String dataIndex = "DEFAULT";
		if (name.indexOf("__") != -1) {
			dataIndex = name.substring(name.indexOf("__") + 2, name.length());
		}
		return myrep.getThaiLabel(this.conversString(value), dataIndex);
	}

	private String conversString(String value) {
		value = value.trim();
		value = value.replace(' ', '-');
		value = value.replace('(', '-');
		value = value.replace(')', '-');
		value = value.replace('#', '-');
		return value;
	}

	public String getPromptOrderLabel(String name, String value, String language) {
		String orderValue = (String) condition.get(value);
		value = (String) myrep.getReportMetaData().getFieldOrderLabel().get(orderValue);
		if (language.equalsIgnoreCase("THA"))
			return this.getPromptThaiLabel(name, value);
		else
			return value;
	}

	public String getFormulaValue(String name) {
		String methodName, value = null;
		methodName = this.formulaMethod.getProperty(name.toUpperCase());
		try {
			if (methodName != null) {
				Class c = this.getClass();
				Method m = c.getDeclaredMethod(methodName, new Class[] { String.class });
				value = (String) m.invoke(this, new Object[] { name });
			}
		} catch (Exception e) {
			MyLog.error(this, " Error in Set FormulaName " + name + " method " + methodName);
		}
		return value;
	}

	private boolean isBeforOnNewPage(int index) {
		String order = (String) condition.get("ORDERBY" + index);
		String newPage = (String) condition.get("ONNEWPAGE");
		int newPageIndex = 0;
		for (int i = 1; i <= myrep.getReportMetaData().getColumnOrderCount(); i++) {
			order = (String) condition.get("ORDERBY" + i);
			if (order != null && order.equals(newPage))
				newPageIndex = i;
		}
		if (index >= newPageIndex)
			return false;
		else
			return true;
	}

	private String getBeforPrintFormula(String s){
		return "timeString";
	}

	private String getAffterPrintFormula(String s){
		String screen=(String)this.condition.get("SCREEN");
		screen=screen.substring(screen.indexOf("-")+1,screen.length());
		return "\""+screen+"-\"+getDateShort()+\"-\"+getTimeShort()";
	}


	private String getOrderIndex(String field) {
		String order = "";
		for (int i = 1; i <= myrep.getReportMetaData().getColumnOrderCount(); i++) {
			order = (String) condition.get("ORDERBY" + i);
			if (order != null && order.equals(field))
				return "ORDERBY" + i;
		}
		return "";
	}

/*	private String getPageNumberFormula(String name) {
		int startPage = Integer.parseInt((String) condition.get("STARTPAGE"));
		String formula = "PageNumber+" + (startPage - 1);
		return formula;
	}
*/

	private String getPageNumberFormula(String name) {
		String page="PAGE";
		String language=(String)this.condition.get("LANGUAGE");
		if(language!=null && language.equals("THA"))
			page=this.getPromptThaiLabel("PAGE","PAGE");
		return "\""+page+"  \" +Totext(PageNumber,0) +\" / \"";
	}


/*	private String getTotalPageCountFormula(String name) {
		int startPage = Integer.parseInt((String) condition.get("STARTPAGE"));
		String formula = "TotalPageCount+" + (startPage - 1);
		return formula;
	}
*/
	private String getOnnewPageFormula(String name) {
		String tableName, fieldName = "";
		String formula = "";
		String newPage = (String) condition.get("ONNEWPAGE");
		if (newPage != null & !newPage.equals("")) {
			formula =
				"if Previous ({"
					+ this.condition.get("SCREEN")
					+ "."
					+ this.getOrderIndex(newPage)
					+ "})={"
					+ this.condition.get("SCREEN")
					+ "."
					+ this.getOrderIndex(newPage)
					+ "} or OnFirstRecord then false else true";
			return formula;
		} else
			return "false";
	}

	private String getOrderByDescFormula(String name) {
		String formula = "{" + (String) myrep.getReportMetaData().getFieldOrderDesc().get(name) + "}";
		if(formula==null){
			return "";
		}
		return formula;
	}

	private String getInvisibleFormular(String name) {
		String dbName=(String)this.condition.get("SCREEN");
		String formula = "IF {" + dbName+"."+name.substring(name.indexOf("_") + 1, name.length()) + "}=\"\" THEN TRUE ELSE FALSE";
		return formula;
	}

	private String getNewPageGroupFormular(String name) {
		int index = Integer.parseInt(name.substring(12, name.length()));
		StringBuffer value = new StringBuffer();
		value = new StringBuffer();
		if (condition.get("ONNEWPAGE") != null && !condition.get("ONNEWPAGE").equals("")) {
			if (index == 1) {
				value.append("if{@OnNewPage}=false  then(");
				value.append("if Previous ({");
				value.append(this.condition.get("SCREEN") + ".ORDERBY1");
				value.append("})<>{");
				value.append(this.condition.get("SCREEN") + ".ORDERBY1");
				value.append("} and OnFirstRecord=false  then true");
				value.append(" else false");
				value.append(")else true");
			} else {
				value.append("if{@OnNewPage}=true then(if ");
				for (int i = index; i > 1; i--) {
					value.append("Previous ({");
					value.append(this.condition.get("SCREEN") + ".ORDERBY" + (i - 1));
					value.append("})<>{");
					value.append(this.condition.get("SCREEN") + ".ORDERBY" + (i - 1));
					value.append("} ");
					if (i > 2)
						value.append(" or ");
				}
				value.append(" then false else true )else");
				if (this.isBeforOnNewPage(index)) {
					value.append("(if ");
					value.append("Previous ({");
					value.append(this.condition.get("SCREEN") + ".ORDERBY" + (index));
					value.append("})<>{");
					value.append(this.condition.get("SCREEN") + ".ORDERBY" + (index));
					value.append("} ");
					value.append(" and ");
					for (int i = index; i > 1; i--) {
						value.append("Previous ({");
						value.append(this.condition.get("SCREEN") + ".ORDERBY" + (i - 1));
						value.append("})={");
						value.append(this.condition.get("SCREEN") + ".ORDERBY" + (i - 1));
						value.append("} ");
						if (i > 2)
							value.append(" or ");
					}
					value.append(" then true else false) ");
				} else
					value.append(" false");
			}
		} else
			return "false";
		return value.toString();
	}

	private String getRecCountFormula(String Name) {
		return "countReccord;";
	}
}
