package com.csc.library.utilities;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Element;

import com.csc.library.session.InitialEnvironment;

/**
 * @author James
 */
public class FieldLabelConfig {
	
	private String configPath;
	
	public static final String MATCH = "MATCH";
	public static final String EXCEPT = "EXCEPT";
	public static final String COMPONENT = "COMPONENT";
	public static final String DESC_THAI = "DESC_THAI";
	public static final String DESC_ENG = "DESC_ENG";
	public static final String ALT_THAI = "ALT_THA";
	public static final String ALT_ENG = "ALT_ENG";

	private HashMap<String,String> fieldMap = new HashMap<String,String>();
	private HashMap<String,List<String>> exceptMap = new HashMap<String,List<String>>();
	private HashMap<String,Map<String,String>> componentMap = new HashMap<String,Map<String,String>>();
	

	//---------------- Construtor ----------------
	public FieldLabelConfig() {
		super();
		setConfigPath(new InitialEnvironment("GLOBAL").getValue("CONFIG-dir") +"FIELD_LABEL_CONFIG.xml");
	}
	
	public void setConfigPath(String path) {
		configPath = path;
	}
	public String getConfigPath(){
		return this.configPath;
	}
	

	public void load() {
		try{
			MyLog.debug("=== Start Load Config XML File: LabelConfig.xml ===");
			ReadXml rm = new ReadXml( getConfigPath() );
			List childs = rm.getRootElement().getChildren();
				
			MyLog.debug("===== Read XML File =====");
			
//			fieldMap.clear();
//			exceptMap.clear();
//			componentMap.clear();
				
			for( Element element : (List<Element>)childs ) {
				String fieldName = element.getAttributeValue("name");
				fieldName = fieldName.trim().toUpperCase();
				// field Listl
				String fieldMatch = element.getChild(MATCH).getText();
				String[] matchArr = fieldMatch.split(",");
				for(String matchStr : matchArr ) {
					matchStr = matchStr.trim().toUpperCase();
					if(matchStr.length()>0) {
						fieldMap.put(matchStr, fieldName);
					}
				}
				
				// Except list
				Element eExcept = element.getChild(EXCEPT);
				if(eExcept != null) {
					String fieldExcept = eExcept.getText();
					String[] exceptArr = fieldExcept.split(",");
					List<String> exceptList = new ArrayList<String>();
					for(String exceptStr : exceptArr ) {
						exceptStr = exceptStr.trim().toUpperCase();
						if(exceptStr.length()>0) {
							exceptList.add(exceptStr);
						}
					}
					exceptMap.put(fieldName, exceptList);
				}
				
				// component map
				Map<String,String> comMap = readComponent(element.getChild(COMPONENT));
				componentMap.put(fieldName, comMap);
				
			}

			MyLog.debug("=== End Read XML File ===");

		}catch(Exception e){
			MyLog.error(this, e);
		}
	}
	private Map<String,String> readComponent(Element component) {
		Map<String,String> map = new HashMap<String,String>();
		if(component != null) {
			String desc_eng = component.getChild("DESC_ENG").getTextTrim();
			String desc_thai = component.getChild("DESC_THAI").getTextTrim();
			String alt_eng = component.getChild("ALT_ENG").getTextTrim();
			String alt_thai = component.getChild("ALT_THAI").getTextTrim();
			
			map.put(DESC_ENG, desc_eng);
			map.put(DESC_THAI, desc_thai);
			map.put(ALT_ENG, alt_eng);
			map.put(ALT_THAI, alt_thai);
		}
		
		return map;
	}
	
	public boolean hasProperties(String tableName, String fieldName) {
		tableName = tableName.trim().toUpperCase();
		fieldName = fieldName.trim().toUpperCase();
		if(fieldMap.containsKey(fieldName) && !isExceptField(tableName,fieldName)) {
			return true;
		}
			
		return false;
	}
	private boolean isExceptField(String tableName, String fieldName) {
		List<String> exceptList = exceptMap.get(fieldMap.get(fieldName));
		String tmp = tableName +"." +fieldName;
		if(exceptList != null && exceptList.contains(tmp)) {
			return true;
		}
		return false;
	}
	
	public Map<String, String> getFieldProperties(String tableName, String fieldName) {
		tableName = tableName.trim().toUpperCase();
		fieldName = fieldName.trim().toUpperCase();
		if(hasProperties(tableName,fieldName)) {
			return componentMap.get(fieldMap.get(fieldName));
		}
		
		return null;

	}

	public static void main(String[] args) {
		FieldLabelConfig config = new FieldLabelConfig();
		String configPath = "D:/EnterprisePT/JbossServices/server/default/deploy/HRPEAP.ear/XML/CONFIGXML/LABELCONFIG.xml";
		config.setConfigPath(configPath);
		try {
			
			config.load();
			
			String tableName = "tadjposition";
			String fieldName = "positionid";
			System.out.println("tableName: " +tableName);
			System.out.println("fieldName: " +fieldName);
			System.out.println();
			boolean hasProperties = config.hasProperties(tableName, fieldName);
			System.out.println("has properties: " +hasProperties);		
			
			if(hasProperties) {
				Map<String,String> map = config.getFieldProperties(tableName, fieldName);
				System.out.println("DESC_ENG: " +map.get(DESC_ENG));
				System.out.println("DESC_THAI: " +map.get(DESC_THAI));
				System.out.println("ALT_ENG: " +map.get(ALT_ENG));
				System.out.println("ALT_THAI: " +map.get(ALT_THAI));
			}

		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}


	


	
}