package com.csc.library.utilities;

import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import com.csc.library.database.MyHashMap;
import com.csc.library.database.StaticProperties;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.InitialInquiry;

/* MyHashMap is HashMap who can sort data by key  */
public class MyTreeMap extends TreeMap {
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long	serialVersionUID	= 1L;
	private UProfile			uprofile			= null;
	public MyTreeMap(UProfile up) {
		super();
		this.setUprofile(up);
		try {
			MyHashMap formulaCompany = (MyHashMap)((MyHashMap) StaticProperties.processData.get(up.getDbName())).get(up.get("companyid"));
			DecimalFormat number = new DecimalFormat("00");
			String[] compen = { "A+", "A-", "B+", "B-", "C+", "C-", "D+", "D-", "E+", "E-" };
			DbInquiry indexInq = new InitialInquiry(this.getUprofile()).getDbInquiry("Mtaxindex", true);
//			Vector incformula = this.getIncomeFormula();
			Vector incformula = (Vector)formulaCompany.get("formulaVector");
			while (indexInq.next()) {
				String taxindex = indexInq.getString("taxindex");
				for (int i = 1; i < 13; i++) {
					for (int j = 0; j < incformula.size(); j++) {
						this.put(taxindex + number.format(i) + "," + incformula.get(j).toString(), "0");
					}
				}
				for (int i = 0; i < compen.length; i++) {
					for (int j = 0; j < incformula.size(); j++) {
						this.put(taxindex + compen[i] + "," + incformula.get(j).toString(), "0");
					}
				}
			}
		} catch (RemoteException e) {
			MyLog.error(this, this.uprofile,  e);		}
	}
	public MyTreeMap() {
		super();
	}
	private Vector getIncomeFormula() {
		Vector result = new Vector();
		try {
			DbInquiry formulaInq = new InitialInquiry(this.getUprofile()).getDbInquiry("Mformula", "((formula_type='1') or (formula_type='2')) and (companyid='" + this.getUprofile().get("companyid") + "')", true);
			while (formulaInq.next()) {
				result.add(formulaInq.getString("formulaid"));
			}
		} catch (Exception e) {
			MyLog.error(this, this.uprofile,  e);
		}
		return result;
	}
	public double getDouble(Object key) {
		String value = (String) this.get(key);
		if (value == null || value.trim().equals("")) {
			return 0;
		} else {
			//this.logger.info("Get value is:"+value);
			return Double.parseDouble(value);
		}
	}	
	
	public Object put(Object key, double value) {
		if (key instanceof String) {
			return super.put((String) key, String.valueOf(value));
		} else {
			return super.put(key,  String.valueOf(value));
		}
	}
	
	public Object put(Object key, Object value) {
		if (key instanceof String) {
			return super.put((String) key, value);
		} else {
			return super.put(key,  value);
		}
	}
	
	public boolean putToSum(MyTreeMap tmap) {
		if (!tmap.isEmpty()) {
			String key = "";
			Iterator it = tmap.keySet().iterator();
			while (it.hasNext()) {
				key = (String) it.next();
				this.putToSum(key, tmap.get(key));
			}
			return true;
		} else {
			return false;
		}
	}
	public void putToSum(Object key, Object value) {
		if (this.containsKey(key)) {
			Object obj = this.get(key);
			if (obj instanceof Double) {
				if (value instanceof Double) {
					Double tmp = (Double) obj;
					this.put(key, new Double(tmp.doubleValue() + ((Double) value).doubleValue()));
				} else {
					MyLog.error(this, getUprofile(),"Type Miss Match Can't sum " + value + " witch Double  Treemap");
				}
			} else if (obj instanceof Integer) {
				if (value instanceof Integer) {
					Integer tmp = (Integer) obj;
					this.put(key, new Integer(tmp.intValue() + ((Integer) value).intValue()));
				} else {
					MyLog.error(this, getUprofile(),"Type Miss Match Can't sum " + value + " witch Integer  Treemap");
				}
			} else if (obj instanceof Float) {
				if (value instanceof Float) {
					Float tmp = (Float) obj;
					this.put(key, new Float(tmp.floatValue() + ((Float) value).floatValue()));
				} else {
					MyLog.error(this, getUprofile(),"Type Miss Match Can't sum " + value + " witch Float  Treemap");
				}
			} else if ((obj instanceof String)) {
				Double tmp = new Double(obj.toString());
				this.put(key, String.valueOf(tmp.doubleValue() + new Double(value.toString()).doubleValue()));
			}
		} else {
			this.put(key, value);
		}
	}
	public void clearValue(Object obj) {
		Iterator it = super.keySet().iterator();
		String key = "";
		while (it.hasNext()) {
			key = (String) it.next();
			super.put(key, obj);
		}
	}
	/**
	 * Returns the uprofile.
	 * @return UProfile
	 */
	public UProfile getUprofile() {
		return uprofile;
	}
	/**
	 * Sets the uprofile.
	 * @param uprofile The uprofile to set
	 */
	public void setUprofile(UProfile uprofile) {
		this.uprofile = uprofile;
	}
}
